/*
** Hello Name Server
*/

#include <cbcp.h>
#define CBCP_NET_TCP_IMPLEMENTATION
#include <cbcp_net_tcp.h>

#include <stdio.h>



/*
** CBCP command predeclarations
*/
static void command_hello_world(CBCP_Command_Args *args);


/*
** CBCP program
*/
int main(int argc, char const *argv[]) {
	if (argc < 2) {
		fprintf(stderr, "Usage:\n\t%s <cbcp_database_filepath>\n", argv[0]);
		return 1;
	}

	/*
	** Initialize CBCP by loading the database file for this host
	*/

	const char *cbcp_database_filepath = argv[1];
	CBCP_State *cbcp;

	cbcp_net_tcp_init(0);
	cbcp = cbcp_init(cbcp_database_filepath);

	if (cbcp == NULL) {
		fprintf(stderr, "Could not initialize CBCP.\n");
		return 1;
	}

	/*
	** Bind command callbacks
	*/

	if (!cbcp_server_init_command(
		cbcp,
		"Hello Name",
		"Hello",
		command_hello_world,
		NULL,
		64)
	) {
		fprintf(stderr, "Could not initialize served command.\n");
		return 1;
	}

	/*
	** Start the cbcp server
	*/

	printf("Starting CBCP server for Hello Name application.\n");

	cbcp_server_start(cbcp);

	return 0;
}

static void command_hello_world(CBCP_Command_Args *args) {
	char *message = (char *)args->payload;
	unsigned int message_length = args->payload_length;

	char *response_payload = (char *)args->response_payload;
	int max_payload_length = args->response_payload_max_length;
	int string_size = snprintf(
		response_payload,
		max_payload_length,
		"Hello, %.*s!",
		message_length,
		message);
	if(string_size < max_payload_length)
		args->response_payload_length = string_size;
	else
		args->response_payload_length = max_payload_length;
	printf(
		"A hello was received from %.*s! Returning the greeting!\n",
		message_length,
		message);
}




